import { Injectable } from '@angular/core';
import { HttpSharedService } from '../http-shared/http-shared.service';
import { apiConfig } from '../config';
import { UrlBuildService } from './url-build.service';
import { catchError, firstValueFrom, map, of } from 'rxjs';
import { BrowserStorageService } from './storage/browser-storage.service';
import { Router } from '@angular/router';

@Injectable({
  providedIn: 'root',
})
export class DesignStudioService {
  userId!: string;
  userDetails!: any;

  constructor(
    private readonly _httpSharedService: HttpSharedService,
    private urlBuildService: UrlBuildService,
    private browserStorageService: BrowserStorageService,
    private router: Router,
  ) {
    this.browserStorageService.getUserDetailsObservable().pipe(
      map((details) => {
        console.log({ details });

        this.userDetails = details;

        return details?.userId;
      }),
    );
  }

  getCloudEnabled() {
    const urlGetCloudEnabledUrl = apiConfig.apiBackendURL + '/api/cloudEnabled';

    return firstValueFrom(
      this._httpSharedService.get(urlGetCloudEnabledUrl).pipe(
        catchError((error) => {
          throw error;
        }),
      ),
    );
  }

  buildURlGetFetchJsonTOSCA(solutionId: string, version: string): string {
    const userId = this.userId;

    // Handle the case where userId is undefined
    if (!userId) {
      throw new Error('User ID is undefined');
    }
    return this.urlBuildService.buildUrl(apiConfig.urlDesignStudioTypeInfo, {
      userId,
      solutionId,
      version,
    });
  }

  httpGet(url: string, params: any) {
    return firstValueFrom(
      this._httpSharedService.get(apiConfig.apiBackendURL + '/' + url, params),
    );
  }

  httpPost(url: string, params: any, body: any) {
    const urlPost = apiConfig.apiBackendURL + '/' + url;
    return firstValueFrom(this._httpSharedService.post(urlPost, params, body));
  }

  redirectTo(path: string) {
    this.router.navigate([path]);
    return firstValueFrom(of(true));
  }
}
